from typing import Generic, Type, TypeVar

T = TypeVar("T")


class Prop(Generic[T]):
    t: Type[T]

    def __new__(cls, t: Type[T], *args, **kwargs):
        cls.t = t
        return super(Prop, cls).__new__(cls, *args, **kwargs)  # type: ignore

    def __init__(self, fget=None, fset=None, fdel=None, doc=None):
        self.fget = fget
        self.fset = fset
        self.fdel = fdel
        if doc is None and fget is not None:
            doc = fget.__doc__
        self.__doc__ = doc

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self
        if self.fget is None:
            raise AttributeError("unreadable attribute")
        return self.fget(obj)

    def __set__(self, obj, value):
        if self.fset is None:
            raise AttributeError("can't set attribute")
        self.fset(obj, value)

    def __delete__(self, obj):
        if self.fdel is None:
            raise AttributeError("can't delete attribute")
        self.fdel(obj)

    def getter(self, fget):
        return type(self)(fget, self.fset, self.fdel, self.__doc__)

    def setter(self, fset):
        return type(self)(self.fget, fset, self.fdel, self.__doc__)

    def deleter(self, fdel):
        return type(self)(self.fget, self.fset, fdel, self.__doc__)



from typing import List

from dapodik.base import BaseDapodik
from . import Validasi


class BaseValidasi(BaseDapodik):
    def sekolah(self, page: int = 1, start: int = 0, limit: int = 50) -> List[Validasi]:
        """Data validasi Sekolah

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("sekolah", page, start, limit)

    def prasarana(
        self, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        """Data validasi Sarpras

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("prasarana", page, start, limit)

    def peserta_didik(
        self, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        """Data validasi Peserta Didik

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("peserta_didik", page, start, limit)

    def ptk(self, page: int = 1, start: int = 0, limit: int = 50) -> List[Validasi]:
        """Data validasi GTK

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("ptk", page, start, limit)

    def rombongan_belajar(
        self, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        """Data validasi Rombongan Belajar & Jadwal

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("rombongan_belajar", page, start, limit)

    def pembelajaran(
        self, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        """Data validasi Pembelajaran

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("pembelajaran", page, start, limit)

    def nilai(self, page: int = 1, start: int = 0, limit: int = 50) -> List[Validasi]:
        """Data validasi Nilai

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("nilai", page, start, limit)

    def referensi(
        self, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        """Data validasi Referensi

        Args:
            page (int, optional): Halaman. Defaults to 1.
            start (int, optional): Dimulai dari. Defaults to 0.
            limit (int, optional): Batas data yang dikembalikan. Defaults to 50.

        Returns:
            List[Validasi]: list dari validasi
        """
        return self._validasi("referensi", page, start, limit)

    def _validasi(
        self, jenis_validasi: str, page: int = 1, start: int = 0, limit: int = 50
    ) -> List[Validasi]:
        params = {
            "jenis_validasi": jenis_validasi,
            "page": page,
            "start": start,
            "limit": limit,
        }
        res = self._get("validation", params=params)
        data: dict = res.json()
        return self._fl(Validasi, data.get("rows"))



from dapodik.base import dataclass

DEF = "Admin.model.PesertaDidikLongitudinal-1"


@dataclass
class PesertaDidikLongitudinal:
    peserta_didik_id: int
    tinggi_badan: int
    berat_badan: int
    lingkar_kepala: int = 0
    peserta_didik_longitudinal_id: int = DEF  # type: ignore
    semester_id: str = "20201"
    jarak_rumah_ke_sekolah_km: int = 1
    jarak_rumah_ke_sekolah: int = 1
    waktu_tempuh_ke_sekolah: int = 0
    menit_tempuh_ke_sekolah: int = 1
    jumlah_saudara_kandung: int = 0
    vld_count: int = 0
    peserta_didik_longitudinal_id_str: str = ""
    peserta_didik_id_str: str = ""
    semester_id_str: str = ""



from dapodik.base import dataclass


@dataclass
class PesertaDidikBaru:
    sekolah_id: str
    nama_pd: str
    jenis_kelamin: str
    tempat_lahir: str
    tanggal_lahir: str
    nama_ibu_kandung: str
    nisn: str = ""
    nik: str = ""
    tahun_ajaran_id: int = 2020
    jenis_pendaftaran_id: int = 1
    sudah_diproses: int = 0
    berhasil_diproses: int = 0
    peserta_didik_id: str = ""
    pdb_id: str = "Admin.model.PesertaDidikBaru-1"
    sekolah_id_str: str = ""
    tahun_ajaran_id_str: str = ""
    jenis_pendaftaran_id_str: str = ""
    peserta_didik_id_str: str = ""



from datetime import datetime, date
from typing import Union


def parse_rows_update(datas: dict, instance):
    data: dict = datas.get("rows", {})
    if data.get(instance._id) == instance._data_id:
        for ke in data:
            setattr(instance, ke, data[ke])
    return instance


def str_to_datetime(
    data: Union[str, datetime], format: str = "%Y-%m-%d %H:%M:%S"
) -> datetime:
    if isinstance(data, datetime):
        return data
    elif type(data) == str:
        return datetime.strptime(data, format)
    else:
        return datetime.now()


def str_to_date(data: Union[str, date], format: str = "%Y-%m-%d") -> date:
    if isinstance(data, date):
        return data
    elif type(data) == str:
        return datetime.strptime(data, format).date()
    else:
        return datetime.now().date()



from requests import Session, Response
from logging import getLogger, Logger
from typing import Any, List, Type, TypeVar

from . import Config, from_dict, from_list


T = TypeVar("T", bound="BaseDapodik")
U = TypeVar("U")


class BaseDapodik:
    def __init__(self, config: Config):
        self._config = None
        self.config = config
        self.__logger: Logger = getLogger("dapodik")
        self.__post_init__()

    def __post_init__(self):
        pass

    @property
    def config(self) -> Config:
        return self._config  # type: ignore

    @config.setter
    def config(self, value: Config):
        if not isinstance(value, Config):
            raise ValueError("config harus berupa dapodik.base.Config")
        self._config = value

    @property
    def username(self) -> str:
        return self.config.username

    @property
    def server(self) -> str:
        return self.config.server

    @property
    def session(self) -> Session:
        return self.config.session

    @property
    def logger(self) -> Logger:
        return self.__logger

    @property
    def semester_id(self) -> str:
        return self.config.semester_id

    def _get(
        self,
        url: str,
        params: dict = None,
        **kwargs: Any,
    ) -> Response:
        return self.session.get(
            self._url(url),
            params=params,
            **kwargs,
        )

    def _get_rest(
        self,
        url: str,
        params: dict = None,
        page: int = 1,
        start: int = 0,
        limit: int = 50,
        prefix: str = "rest/",
        **kwargs: Any,
    ) -> Response:
        params_ = {
            "page": page,
            "start": start,
            "limit": limit,
        }
        if params:
            params_.update(params)
        return self.session.get(self._url(prefix + url), params=params_, **kwargs)

    def _post(
        self,
        url: str,
        data: dict = None,
        json: dict = None,
        params: dict = None,
        **kwargs: Any,
    ) -> Response:
        return self.session.post(
            self._url(url),
            data=data,
            json=json,
            params=params,
            **kwargs,
        )

    def _url(self, url: str) -> str:
        if not url.startswith(self.server):
            return self.server + url.lstrip("/")
        return url

    def _fd(self, t: Type[U], data: dict) -> U:
        return from_dict(t, data)

    def _fl(self, t: Type[U], datas: List[dict] = None) -> List[U]:
        return from_list(t, datas) if datas else list()



from typing import Any, List, Optional
from dapodik.base import dataclass, field


@dataclass
class PesertaDidik:
    nama: str
    jenis_kelamin: str
    tempat_lahir: str
    tanggal_lahir: str
    nama_ibu_kandung: str
    sekolah_id: str
    pdb_id: str
    tahun_lahir_ayah: int
    tahun_lahir_ibu: int
    agama_id: int
    alamat_jalan: str
    rt: int
    rw: int
    nama_dusun: str
    desa_kelurahan: str
    kode_wilayah: str
    kode_wilayah_str: str
    kode_pos: str
    lintang: int = 0
    bujur: int = 0
    nisn: str = ""
    nik: str = ""
    tahun_lahir_wali: int = 1980
    vld_count: int = 0
    kewarganegaraan: str = "ID"
    peserta_didik_id: str = "Admin.model.PesertaDidik-1"
    kewarganegaraan_str: str = ""
    no_kk: str = ""
    reg_akta_lahir: str = ""
    agama_id_str: str = ""
    kebutuhan_khusus_id: int = 0
    kebutuhan_khusus_id_str: str = ""
    jenis_tinggal_id: int = 1
    alat_transportasi_id: int = 1
    jenis_tinggal_id_str: str = ""
    alat_transportasi_id_str: str = ""
    no_kks: str = ""
    anak_keberapa: int = 1
    penerima_kps: int = 0
    no_kps: str = ""
    penerima_kip: int = 0
    no_kip: str = ""
    nm_kip: str = ""
    layak_pip: int = 0
    id_layak_pip: Optional[int] = None
    id_layak_pip_str: str = ""
    id_bank: str = ""
    id_bank_str: str = ""
    rekening_bank: str = ""
    nama_kcp: str = ""
    rekening_atas_nama: str = ""
    status_data: int = 0
    nama_ayah: str = ""
    nik_ayah: str = ""
    jenjang_pendidikan_ayah: int = 0
    jenjang_pendidikan_ayah_str: str = ""
    pekerjaan_id_ayah: int = 0
    pekerjaan_id_ayah_str: str = ""
    penghasilan_id_ayah: int = 0
    penghasilan_id_ayah_str: str = ""
    kebutuhan_khusus_id_ayah: int = 0
    kebutuhan_khusus_id_ayah_str: str = ""
    nik_ibu: str = ""
    jenjang_pendidikan_ibu: int = 0
    jenjang_pendidikan_ibu_str: str = ""
    pekerjaan_id_ibu: int = 1
    pekerjaan_id_ibu_str: str = ""
    penghasilan_id_ibu: int = 99
    penghasilan_id_ibu_str: str = ""
    kebutuhan_khusus_id_ibu: int = 0
    kebutuhan_khusus_id_ibu_str: str = ""
    nama_wali: str = ""
    nik_wali: str = ""
    jenjang_pendidikan_wali: int = 0
    jenjang_pendidikan_wali_str: str = ""
    pekerjaan_id_wali: int = 0
    pekerjaan_id_wali_str: str = ""
    penghasilan_id_wali: int = 0
    penghasilan_id_wali_str: str = ""
    nomor_telepon_rumah: str = ""
    nomor_telepon_seluler: str = ""
    email: str = ""
    kebutuhan_khusus_id_selector: List[Any] = field(factory=list)
    kebutuhan_khusus_id_selector_ayah: List[Any] = field(factory=list)
    kebutuhan_khusus_id_selector_ibu: List[Any] = field(factory=list)



from functools import wraps


def lazy(func):
    """Decorator that makes a property lazy-evaluated.
    Source : https://stevenloria.com/lazy-properties/
    """
    attr_name = "_lazy_" + func.__name__

    @wraps(func)
    def wrapper(self):
        if not hasattr(self, attr_name):
            setattr(self, attr_name, func(self))
        return getattr(self, attr_name)

    return wrapper



from datetime import datetime
from typing import Optional

from dapodik.base import dataclass


@dataclass(frozen=True)
class Jurusan:
    jurusan_id: str
    nama_jurusan: str
    untuk_sma: str
    untuk_smk: str
    untuk_pt: str
    untuk_slb: str
    untuk_smklb: str
    jenjang_pendidikan_id: Optional[str]
    jurusan_induk: Optional[str]
    level_bidang_id: str
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime

    @property
    def level_bidang(self):
        # TODO API
        return self.level_bidang_id



from typing import List, Type, TypeVar


T = TypeVar("T")


def from_dict(t: Type[T], data: dict) -> T:
    return t(**data)  # type: ignore


def from_list(t: Type[T], datas: List[dict]) -> List[T]:
    results: List[T] = list()
    for data in datas:
        results.append(from_dict(t, data))
    return results



from datetime import datetime
from typing import Optional

from dapodik.base import dataclass


@dataclass(frozen=True)
class KategoriTk:
    kategori_tk_id: str
    nama: str
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime



from dapodik.base import BaseDapodik
from . import Sekolah


class BaseSekolah(BaseDapodik):
    def __call__(self) -> Sekolah:
        return self.sekolah()

    def sekolah(self) -> Sekolah:
        res = self._get_rest("Sekolah")
        data: dict = res.json()
        _sekolah: list = data.get("rows", list())
        return self._fd(Sekolah, _sekolah[0])



#!/usr/bin/env python

from .version import __version__, __dapodik_version__, __semester__  # NOQA

from .auth import BaseAuth
from .sekolah import BaseSekolah
from .validasi import BaseValidasi

from .dapodik import Dapodik

__all__ = [
    "BaseAuth",
    "BaseSekolah",
    "BaseValidasi",
    "Dapodik",
]



from .config import HEADERS
from .attrs import asdict, dataclass, sdataclass, field, fields, freeze
from .utils import from_dict, from_list
from .config import Config
from .prop import Prop
from .dapodik import BaseDapodik

__all__ = [
    "HEADERS",
    "asdict",
    "dataclass",
    "sdataclass",
    "field",
    "fields",
    "freeze",
    "from_dict",
    "from_list",
    "Config",
    "Prop",
    "BaseDapodik",
]



from datetime import datetime, date
from typing import Optional

from dapodik.base import dataclass, freeze


@dataclass
class Sekolah:
    sekolah_id: str = freeze()
    nama: str = freeze()
    nama_nomenklatur: Optional[str] = freeze()
    nss: str = freeze()
    npsn: str = freeze()
    bentuk_pendidikan_id: int = freeze()
    alamat_jalan: str = freeze()
    rt: Optional[str]
    rw: Optional[str]
    nama_dusun: Optional[str]
    desa_kelurahan: str = freeze()
    kode_wilayah: str = freeze()
    kode_pos: Optional[str]
    lintang: Optional[str]
    bujur: Optional[str]
    nomor_telepon: Optional[str]
    nomor_fax: Optional[str]
    email: Optional[str]
    website: Optional[str]
    kebutuhan_khusus_id: int
    status_sekolah: str = freeze()
    sk_pendirian_sekolah: str = freeze()
    tanggal_sk_pendirian: date = freeze()
    status_kepemilikan_id: str = freeze()
    yayasan_id: str
    sk_izin_operasional: str = freeze()
    tanggal_sk_izin_operasional: date = freeze()
    no_rekening: Optional[str] = freeze()
    nama_bank: Optional[str] = freeze()
    cabang_kcp_unit: Optional[str] = freeze()
    rekening_atas_nama: Optional[str] = freeze()
    mbs: str
    luas_tanah_milik: str = freeze()
    luas_tanah_bukan_milik: str = freeze()
    kode_registrasi: str = freeze()
    npwp: Optional[str] = None
    nm_wp: Optional[str] = None
    keaktifan: str = freeze()
    flag: Optional[str] = freeze()
    create_date: datetime = freeze()
    last_update: datetime = freeze()
    soft_delete: str = freeze()
    last_sync: datetime = freeze()
    updater_id: str = freeze()
    bentuk_pendidikan_id_str: str
    kode_wilayah_str: str = freeze()
    kebutuhan_khusus_id_str: str = freeze()
    yayasan_id_str: str = freeze()
    vld_count: int = freeze()



import attr
from datetime import datetime, date
from functools import partial, wraps
from typing import Any, Callable, List, Optional

from dapodik.utils.parser import str_to_datetime, str_to_date

COMMON: List = [str, int, datetime]


def hooq(cls: type, fields: List[attr.Attribute]) -> List[attr.Attribute]:
    results: List[attr.Attribute] = list()
    return results


def frozen(_, __, ___):
    """
    Prevent an attribute to be modified.

    .. versionadded:: 20.1.0
    """
    raise attr.exceptions.FrozenAttributeError()


def serialize(
    inst=None, field: attr.Attribute = None, value: Any = None, name: str = None
) -> Any:
    if not value:
        return value
    if not name:
        name = getattr(field, "name", "")
    if isinstance(value, list):
        out = {}
        for idx, val in enumerate(value):
            val = serialize(value=val)
            if isinstance(val, dict):
                for key, value in val.items():
                    out[f"{name}[{idx}][{key}]"] = val[key]
            else:
                out_key = name or ""
                # Check if data required name prefix
                if hasattr(value, "name"):
                    out_key += f"[{getattr(value, 'name')}]"
                out_key += f"[{idx}]"
                out[out_key] = val
        return out
    elif isinstance(value, dict):
        out = {}
        for key, value in value.items():
            if isinstance(value, list):
                out.update(serialize(value=val, name=key))
            else:
                out[key] = value
        return out
    elif attr.has(value):
        return attr.asdict(value, value_serializer=serialize)  # type: ignore
    elif isinstance(value, datetime):
        return datetime.strftime(value, "%Y-%m-%d %H:%M:%S")
    elif isinstance(value, date):
        return date.strftime(value, "%Y-%m-%d")
    return value


def auto_convert(cls, fields):
    results = []
    for field in fields:
        if field.converter is not None:
            results.append(field)
            continue
        if field.type in {datetime, "datetime"}:
            converter = str_to_datetime
        elif field.type in {date, "date"}:
            converter = str_to_date
        elif field.type == Optional[field.type]:

            def converter(d=None):
                return d

        else:
            converter = None
        results.append(field.evolve(converter=converter))
    return results


asdict: Callable = wraps(attr.asdict)(
    partial(
        attr.asdict,
        value_serializer=serialize,
    )
)
dataclass: Callable = wraps(attr.attrs)(
    partial(
        attr.attrs,
        auto_attribs=True,
        field_transformer=auto_convert,
        kw_only=True,
    )
)
sdataclass: Callable = wraps(attr.attrs)(
    partial(
        attr.attrs,
        auto_attribs=True,
        field_transformer=auto_convert,
        slots=True,
        kw_only=True,
    )
)
field = attr.attrib
freeze = wraps(attr.attrib)(partial(attr.attrib, on_setattr=frozen))


def fields(converter: Callable, **kwargs):
    def conv(d: List[dict]):
        results: List = list()
        if not isinstance(d, list):
            return results
        for result in results:
            results.append(converter(**result))
        return results

    return attr.attrib(converter=conv, factory=list, **kwargs)



from datetime import datetime
from typing import Optional

from dapodik.base import dataclass


@dataclass(frozen=True)
class Penghasilan:
    penghasilan_id: int
    nama: str
    batas_bawah: int
    batas_atas: int
    create_date: datetime
    last_update: datetime
    expired_date: Optional[datetime]
    last_sync: datetime



from requests import Session

from dapodik import __semester__
from dapodik.base import BaseDapodik, HEADERS, Config
from dapodik.utils.decorator import lazy

from . import BaseAuth
from . import BaseSekolah
from . import BaseValidasi


class Dapodik(BaseDapodik):
    def __init__(
        self,
        username: str,
        password: str,
        semester_id: str = __semester__,
        server: str = "http://localhost:5774/",
        session: Session = Session(),
        pengguna: int = 0,
        rememberme: bool = True,
    ):
        config = Config(username, password, server, semester_id)
        super(Dapodik, self).__init__(config)
        self.session.headers.update(HEADERS)
        self.daftar_pengguna = self.auth.login(
            username, password, rememberme, semester_id, pengguna
        )

    @property  # type: ignore
    @lazy
    def auth(self) -> BaseAuth:
        return BaseAuth(self.config)

    @property  # type: ignore
    @lazy
    def sekolah(self) -> BaseSekolah:
        return BaseSekolah(self.config)

    @property  # type: ignore
    @lazy
    def validasi(self) -> BaseValidasi:
        return BaseValidasi(self.config)

    def __del__(self):
        self.auth.logout()



from requests import Session
from typing import Optional

from dapodik import __semester__

HEADERS = {
    "DNT": "1",
    "Referer": "http://localhost:5774/",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:85.0) Gecko/20100101 Firefox/85.0",
}


class Config:
    def __init__(
        self,
        username: str,
        password: str,
        server: str = "http://localhost:5774/",
        semester_id: str = __semester__,
        sekolah_id: Optional[str] = None,
        session: Optional[Session] = None,
    ):
        f"""Config

        Args:
            username (str): Email dapodik
            password (str): Password dapodik
            server (str, optional): Url server dapodik. Default "http://localhost:5774/".
            semester_id (str, optional): ID semester. Default {__semester__}.
            sekolah_id (Optional[str], optional): ID sekolah. Default None.
            session (Optional[Session], optional): Sesi. Default None.
        """
        self._username = username
        self._password = password
        self._server = server.rstrip("/") + "/"
        self._semester_id = semester_id
        self._sekolah_id = sekolah_id
        if isinstance(session, Session):
            self._session = session
        else:
            self._session = Session()
        self.session.headers.update(HEADERS)

    @property
    def username(self) -> str:
        return self._username

    @property
    def password(self) -> str:
        return self._password

    @property
    def server(self) -> str:
        return self._server

    @property
    def semester_id(self) -> str:
        return self._semester_id

    @property
    def sekolah_id(self) -> Optional[str]:
        return self._sekolah_id

    @sekolah_id.setter
    def sekolah_id(self, value: str):
        if not isinstance(value, str):
            raise ValueError("sekolah_id harus berupa string")
        self._sekolah_id = value

    @property
    def session(self) -> Session:
        return self._session

    @session.setter
    def session(self, value: Session):
        if not isinstance(value, Session):
            raise ValueError("session harus instance dari requests.Session")
        self._session = value
